﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using BlacksmithWorkshopContracts.BindingModels;
using BlacksmithWorkshopContracts.BusinessLogicsContracts;
using BlacksmithWorkshopContracts.SearchModels;
using BlacksmithWorkshopContracts.StoragesContracts;
using BlacksmithWorkshopContracts.ViewModels;
using BlacksmithWorkshopDataModels.Enums;
using Microsoft.Extensions.Logging;

namespace BlacksmithWorkshopBusinessLogic.BusinessLogics
{
    public class OrderLogic : IOrderLogic
    {
        private readonly ILogger _logger;
        private readonly IOrderStorage _orderStorage;
		private readonly IShopLogic _shopLogic;
		private readonly IManufactureStorage _manufactureStorage;
		public OrderLogic(ILogger<OrderLogic> logger, IOrderStorage orderStorage, IShopLogic shopLogic, IManufactureStorage manufactureStorage)
        {
            _logger = logger;
            _orderStorage = orderStorage;
			_shopLogic = shopLogic;
			_manufactureStorage = manufactureStorage;
		}
        public List<OrderViewModel>? ReadList(OrderSearchModel? model)
        {
            _logger.LogInformation("ReadList. OrderId:{Id}", model?.Id);
            var list = model == null ? _orderStorage.GetFullList() : _orderStorage.GetFilteredList(model);
            if (list == null)
            {
                _logger.LogWarning("ReadList return null list");
                return null;
            }
            _logger.LogInformation("ReadList. Count:{Count}", list.Count);
            return list;
        }
        public bool CreateOrder(OrderBindingModel model)
        {
			CheckModel(model);
			if (model.Status != OrderStatus.Неизвестен)
			{
				_logger.LogWarning("Insert operation failed. Order status is incorrect.");
				return false;
			}
			model.Status = OrderStatus.Принят;
			if (_orderStorage.Insert(model) == null)
			{
				model.Status = OrderStatus.Неизвестен;
				_logger.LogWarning("Insert operation failed");
				return false;
			}
			return true;
		}
        public bool TakeOrderInWork(OrderBindingModel model)
        {
			return StatusUpdate(model, OrderStatus.Выполняется);
		}
        public bool DeliveryOrder(OrderBindingModel model)
        {
			return StatusUpdate(model, OrderStatus.Выдан);
		}
        public bool FinishOrder(OrderBindingModel model)
        {
			return StatusUpdate(model, OrderStatus.Готов);
		}
        private void CheckModel(OrderBindingModel model, bool withParams = true)
        {
			if (model == null)
			{
				throw new ArgumentNullException(nameof(model));
			}
			if (!withParams)
			{
				return;
			}
			if (model.ManufactureId < 0)
			{
				throw new ArgumentNullException("Некорректный идентификатор компьютера", nameof(model.ManufactureId));
			}
			if (model.Count <= 0)
			{
				throw new ArgumentNullException("Количество компьютеров в заказе должно быть больше 0", nameof(model.Count));
			}
			if (model.Sum <= 0)
			{
				throw new ArgumentNullException("Сумма заказа должна быть больше 0", nameof(model.Sum));
			}
			_logger.LogInformation("Order. OrderId: {Id}.Sum: {Sum}. ManufactureId: {ManufactureId}", model.Id, model.Sum, model.ManufactureId);
		}
		private bool StatusUpdate(OrderBindingModel model, OrderStatus newStatus)
		{
			var viewModel = _orderStorage.GetElement(new OrderSearchModel { Id = model.Id });
			if (viewModel == null)
			{
				throw new ArgumentNullException(nameof(model));
			}
			if (viewModel.Status + 1 != newStatus)
			{
				_logger.LogWarning("Change status operation failed");
				return false;
			}
			model.Status = newStatus;
			if (model.Status == OrderStatus.Готов)
			{
				model.DateImplement = DateTime.Now;
				var manufacture = _manufactureStorage.GetElement(new() { Id = viewModel.ManufactureId });
				if (manufacture == null)
				{
					throw new ArgumentNullException(nameof(manufacture));
				}
				if (!_shopLogic.AddManufactures(manufacture, viewModel.Count))
				{
					throw new Exception($"AddManufactures operation failed");
				}
			}
			else
			{
				model.DateImplement = viewModel.DateImplement;
			}
			CheckModel(model, false);
			if (_orderStorage.Update(model) == null)
			{
				_logger.LogWarning("Change status operation failed");
				return false;
			}
			return true;
		}
	}
}
}
